require 'sequel'
require 'storing/abstract/abstract_adaptor'

module Storing
	module Postgres
		class Adaptor < ::Storing::AbstractAdaptor
			attr_reader :database

			def self.connect *args
				new Sequel.postgres(*args)
			end

			def initialize database
				@database = database
			end

			def query query_object
				database[query_object.query(database)].all
			end

			def select table_name, conditions={}
				database[table_name].where(conditions).all
			end

			def insert table_name, params={}
				database[table_name].insert(params)
			end

			def update table_name, params={}, conditions={}
				database[table_name].where(conditions).update(params)
			end

			def delete table_name, conditions={}
				database[table_name].where(conditions).delete
			end

			def transaction &block
				database.transaction &block
			end
		end
	end
end