require 'storing/abstract/abstract_store'
require 'storing/exceptions'

module Storing
	class Store < AbstractStore
		class Entity
			attr_reader :primary_key_column, :data

			def initialize primary_key_column, data={}
				@primary_key_column = primary_key_column
				@data = data
			end

			def primary_key_hash
				primary_key_columns.inject({}) do |memo, pk_col|
					memo[pk_col] = data.fetch(pk_col)
					memo
				end
			rescue KeyError => e
				raise(MissingPrimaryKey, 'primary key missing in entity data hash')
			end

			def params
				data.reject{|k,v| primary_key_columns.include? k }
			end

			private

			def primary_key_columns
				[*primary_key_column]
			end
		end

		def self.set_primary_key val
			@primary_key = val
		end

		def self.primary_key
			@primary_key || raise(MissingPrimaryKey, 'primary key not assigned')
		end

		def self.set_table_name val
			@table_name = val
		end

		def self.table_name
			@table_name || raise(MissingTableName, 'table name not assigned')
		end

		attr_reader :adaptor

		def initialize adaptor
			@adaptor = adaptor
		end

		def query query_object
			adaptor.query(query_object)
		end

		def select conditions={}
			adaptor.select(table_name, conditions)
		end

		def insert entity_hash={}
			adaptor.insert(table_name, entity_hash)
		end

		def update entity_hash={}
			entity = new_entity(entity_hash)
			adaptor.update(table_name, entity.params, entity.primary_key_hash)
		end

		def delete entity_hash={}
			entity = new_entity(entity_hash)
			adaptor.delete(table_name, entity.primary_key_hash)
		end

		def transaction
			adaptor.transaction
		end

		def table_name
			self.class.table_name
		end

		def primary_key
			self.class.primary_key
		end

		private

		def new_entity entity_hash={}
			::Storing::Store::Entity.new(primary_key, entity_hash)
		end
	end
end